#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/09/2002
#################################################################################
#
# Dice - Script part of Webchat designed to send dice rolls
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Comm;						# Communication handler
use Ethereal::Database;						# Database handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handling
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table handler


#################################################################################
# Data Members
#################################################################################
my $comm;							# Communications panel handle
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %realm;							# Realm hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);	

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Login and Comm handle
	$login    = new Ethereal::Login($database, $cgi, $param); 
	$comm     = new Ethereal::Comm($database, $cgi);

	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Prevent bans from use
		$login->CheckLegality(\%realm);

		# Make sure there is a color
		unless (defined($param->{'COLOR'}))
		{
			# Determine and assign
			my ($tsize, $tcolor, $tface) = $database->DataGetFont($param->{'CHAR'});
			$param->{'COLOR'} = $tcolor;
			$param->{'FACE'}  = $tface;
		}

		# Enforce size
		$param->{'SIZE'} = 3;


		# Only do the following if posting is needed
		if (defined($param->{'ROLL'}))
		{
			RollHandler($param, \%realm);
			$comm->DataWrite($param);
		}

		# Display panel
		DiceDisplay($database, $cgi, $param, \%realm);
	}


#################################################################################
# Sub-Routines
#################################################################################

####################
# Dice Display
#
# Gives the users options for the roll of dice.  This means the selection of the
# type of die as well as the number of specific rolls.

sub DiceDisplay
{
	####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# Cgi handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Chat options

	my $option   = new Ethereal::Option($cgi);		# Option handler

	my $btch;						# Botches
	my $diff;						# Difficulty
	my $damg;						# Damage modifier
	my $type;						# Selection type
	my $freq;						# Frequency

	my $panel;						# Dice roller panel

	my $dlowr = $param->{'DLOWR'};				# Drop lowest
	my $dhigh = $param->{'DHIGH'};				# Drop highest

	my $dbtch = $param->{'DBTCH'};				# Damage modifier
	my $ddamg = $param->{'DDAMG'};				# Damage modifier
	my $ddiff = $param->{'DDIFF'};				# Dice difficulty
	my $dfreq = $param->{'DFREQ'};				# Dice frequency
	my $dtype = $param->{'DTYPE'};				# Dice type

	my @types;						# Dice types


	####################
	# Program area

	# Find types
	$option->Split(\@types, $realm->{'OptDiceTypes'});

	# Assign values
	$btch = (defined($dbtch)) ? $dbtch : 0;
	$damg = (defined($ddamg)) ? $ddamg : 0;
	$diff = (defined($ddiff)) ? $ddiff : 0;
	$freq = (defined($dfreq)) ? $dfreq : 1;
	$type = (defined($dtype)) ? $dtype : $types[0];

	$dlowr = (defined($dlowr)) ? $dlowr : 0;
	$dhigh = (defined($dhigh)) ? $dhigh : 0;	


	
        # Retreive dice roller panel
        $panel = $database->FetchRealmDice($param->{'ROOM'});

        # Widgets
        # Assigning values
        my $wdiceroll = $cgi->submit($realm->{'TxtRoll'});

	# Basics
        my $wdicefreq = $cgi->textfield('DFREQ', $freq, 3, 2);
        my $wdicetype = $cgi->popup_menu('DTYPE',  \@types,  $type); 

	# Success
	my $wdicediff = $cgi->textfield('DDIFF', $diff, 3, 2);
	my $wdicebtch = $cgi->textfield('DBTCH', $btch, 3, 2);

	# Damage modifer
	my $wdicedamg = $cgi->textfield('DDAMG', $damg, 3, 2);
	
	# Drops
	my $wdicelowr = $cgi->textfield('DLOWR', $dlowr, 3, 2);
	my $wdicehigh = $cgi->textfield('DHIGH', $dhigh, 3, 2);


	# Form values
	my $wformstrt = $cgi->start_form();
	my $wformhidn = $param->EmbedNormal($cgi, %{$param}, 'ROLL', 'True');
	my $wformends = $cgi->end_form();


	# Basics
	$panel =~ s/WDICEFREQ/$wdicefreq/gs;
	$panel =~ s/WDICETYPE/$wdicetype/gs;

	# Success
	$panel =~ s/WDICEDIFF/$wdicediff/gs;
	$panel =~ s/WDICEBTCH/$wdicebtch/gs;

	# Damage modifier
	$panel =~ s/WDICEDAMG/$wdicedamg/gs;

	# Drops
	$panel =~ s/WDICELOWR/$wdicelowr/gs;
	$panel =~ s/WDICEHIGH/$wdicehigh/gs;

	# Submit
	$panel =~ s/WDICEROLL/$wdiceroll/gs;

	# Form
	$panel =~ s/WFORMSTRT/$wformstrt$wformhidn/gs;
	$panel =~ s/WFORMENDS/$wformends/gs;



	# Display HTML
	# Document header
	print $cgi->header;

	# Document
	print "$panel";
}


####################
# Roll Handler
#
# Will perform a roll of the dice as specified by the user himself.   This is done
# using a pseudo random seed.

sub RollHandler							# Database, Cgi, Param
{
	####################
	# Data members
	my $param = shift;					# Parameter handle
	my $realm = shift;					# Chat options

	my $mes;						# Simple message
	my $result;						# Singular result
	my $results;						# Formatted results
	my $total;						# Active total

	my $botches;						# Number of botches
	my $damage;						# Damage modifier string
	my $count;						# Counter
	my $success;						# Number of sucesses
	my $rand;						# Random result

	my @results;						# Array of results


	####################
	# Program area

	# Adjust seed
	srand();

	# Ensure defined
	$param->{'DBTCH'} = (defined($param->{'DBTCH'})) ? $param->{'DBTCH'} : 1;
	$param->{'DDAMG'} = (defined($param->{'DDAMG'})) ? $param->{'DDAMG'} : 0;
	$param->{'DDIFF'} = (defined($param->{'DDIFF'})) ? $param->{'DDIFF'} : 0;
	$param->{'DFREQ'} = (defined($param->{'DFREQ'})) ? $param->{'DFREQ'} : 1;
	$param->{'DLOWR'} = (defined($param->{'DLOWR'})) ? $param->{'DLOWR'} : 0;
	$param->{'DHIGH'} = (defined($param->{'DHIGH'})) ? $param->{'DHIGH'} : 0;

	# Make sure numbers are received
	$param->{'DBTCH'} = ($param->{'DBTCH'} =~ /^\d+$/) ? $param->{'DBTCH'} : 1;
	$param->{'DDAMG'} = ($param->{'DDAMG'} =~ /^(-|\d)+$/) ? $param->{'DDAMG'} : 0;
	$param->{'DDIFF'} = ($param->{'DDIFF'} =~ /^\d+$/) ? $param->{'DDIFF'} : 0;
	$param->{'DFREQ'} = ($param->{'DFREQ'} =~ /^\d+$/) ? $param->{'DFREQ'} : 1;
	$param->{'DLOWR'} = ($param->{'DLOWR'} =~ /^\d+$/) ? $param->{'DLOWR'} : 0;
	$param->{'DHIGH'} = ($param->{'DHIGH'} =~ /^\d+$/) ? $param->{'DHIGH'} : 0;

	# Number cap
	$param->{'DFREQ'} = ($param->{'DFREQ'} < 250) ? $param->{'DFREQ'} : 250;
	$param->{'DDAMG'} = ($param->{'DDAMG'} < 50)  ? $param->{'DDAMG'} :  50;

	# Difficulty cannot surpass type
	$param->{'DDIFF'} = ($param->{'DDIFF'} > $param->{'DTYPE'}) ? 0 : $param->{'DDIFF'};

	# Botches cannot be higher then diff
	$param->{'DBTCH'} = ($param->{'DBTCH'} < $param->{'DDIFF'}) ? $param->{'DBTCH'} : 0;


	# Drops cannot surpass dice
	if (($param->{'DLOWR'} + $param->{'DHIGH'}) > ($param->{'DFREQ'} - 1))
	{
		# Reset them to zero
		$param->{'DLOWR'} = 0;
		$param->{'DHIGH'} = 0;
	}


	# Damage modifer handling
	if ($param->{'DDAMG'} != 0)
	{
		# Positive modifer
		if ($param->{'DDAMG'} > 0)
		{ $damage = "+$param->{DDAMG}"; }

		# Negative modifier
		elsif ($param->{'DDAMG'} < 0)
		{ $damage = "$param->{DDAMG}"; }

		# Simply spacing
		else
		{ $damage = ''; }
	}


	# Comments
	if ((defined($param->{'DPOST'}))
	 && (length($param->{'DPOST'}) > 3))
	{
		# Append comments
		$results = "<B>" . $realm->{'TxtDiceAction'} . "</B> " . $param->{'DPOST'} . "<BR>\n";
	}
	else
	{
		# Empty string
		$results = '';
	} 

	# Contruct begin results
	$rand    = int((rand() * $param->{'DTYPE'}) + 1);
	$results = $results . "<B>" . $realm->{TxtDiceRoll} . " </B>(<B>$param->{DFREQ}d$param->{DTYPE}$damage</B>) " . $rand;
	$total  += $rand + $param->{'DDAMG'};

	# Append to array
	push(@results, $rand);

	# Rolls for each roll
	for ($count = 1; $count < $param->{'DFREQ'}; $count++)
	{
		# Generate number
		$rand = int((rand() * $param->{'DTYPE'}) + 1);


		# Continue contruct
		$results = "$results, $rand";

		# Work on total
		$total += $rand;

		# Append to array
		push(@results, $rand);
	} 

	# Sort array
	@results = sort Numeric @results;

	
	# Initial Results
	$results = "$results ($total)<BR>";

	# Drops
	# Lowest
	if ($param->{'DLOWR'} > 0)
	{
		# Reset counter
		$count = 1;


		# Pull initial result
		$result = shift(@results);
		$total -= $result;

		# Format
		$results = $results . "<BR><B>$realm->{TxtDiceDrop} " . $param->{'DLOWR'} . " $realm->{TxtDiceHigh}</B>: " . $result;


		# Loop until out of elements or reached count
		while (($result = shift(@results)) and ($count < $param->{'DLOWR'}))
		{
			# Drop from result
			$total -= $result;

			# Massage string
			$results = $results . ", $result";

			# Increment counter
			$count++;
		}

		# Final format
		$results = $results . " (" . $total . ")";
	}

	# Highest
	if ($param->{'DHIGH'} > 0)
	{
		# Reset counter
		$count = 1;


		# Pull initial result
		$result = pop(@results);
		$total -= $result;

		# Format
		$results = $results . "<BR><B>$realm->{TxtDiceDrop} " . $param->{'DHIGH'} . " $realm->{TxtDiceHigh}</B>: " . $result;


		# Loop until out of elements or reached count
		while (($result = pop(@results)) and ($count < $param->{'DHIGH'}))
		{
			# Drop from result
			$total -= $result;

			# Massage string
			$results = $results . ", $result";

			# Increment counter
			$count++;
		}

		# Final format
		$results = $results . " (" . $total . ")";
	}


	# Botches and sucesses
	if ($param->{'DDIFF'} > 0)
	{
		# Additional padding
		$results = (($param->{'DHIGH'} > 0) or ($param->{'DLOWR'} > 0)) ? "$results<BR>" : $results;

		# Determine amounts
		while ($result = shift(@results))
		{
			# Success and botches
			$success += ($result > $param->{'DDIFF'}) ? 1 : 0;
			$botches += ($result < ($param->{'DBTCH'}) + 1) ? 1 : 0;
		}


		# Botches
		if ($param->{'DBTCH'} > 0)
		{
			# Botch
			if ($botches > $success)
			{ $mes = "<B>" . $realm->{'TxtDiceBotch'} . "</B> ($botches)" ; }

			# Success
			elsif ($botches < $success)
			{ $mes = "<B>" . $realm->{'TxtDiceSuccess'} . "</B> ($success)"; }

			# Failed
			else
			{ $mes = "<B>" . $realm->{'TxtDiceFail'} . "</B>"; }
		}

		# Simple success
		else
		{
			# Successful message
			$mes = "<B>" . $realm->{'TxtDiceSuccess'} . "</B> ($success)";
		}

		# Display message
		$results = $results . "<BR>$mes";
	}


	# Assign contruct as post
	$param->{'POST'}   = $results;
	$param->{'TARGET'} = 'dice roll';
}


####################
# Numeric
#
# Numeric comparison in order to be able to sort by numbers.   

sub Numeric
{ $a <=> $b; } 
